<?php

namespace SCM\Services;

use SCM\Traits\TQuery;

class Cardapio {
    
    use TQuery;

    private $entityManager;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function retornaCardapiosDatas($centrais, $dtInicial, $dtFinal, $nrorg) {
        $sql = "SELECT CDFILCENTPME, NRCARDPME, DSCARDPME, DTINICARDPME, DTFINCARDPME, CDTIPOCARD
          FROM CARDAPIOPME C
         WHERE 
         INSTR(:CDFILCENTPME, C.CDFILCENTPME) <> 0
         AND NRORG = :NRORG
           AND (C.DTINICARDPME BETWEEN :DTINI AND :DTFIM
           OR C.DTFINCARDPME BETWEEN :DTINI AND :DTFIM
           OR :DTINI BETWEEN C.DTINICARDPME AND C.DTFINCARDPME
           OR :DTFIM BETWEEN C.DTINICARDPME AND C.DTFINCARDPME)";

        $params = array(
            'NRORG' => $nrorg,
            'CDFILCENTPME' => $centrais,
            'DTINI' => $dtInicial,
            'DTFIM' => $dtFinal,
        );
        $cardapiosDatas = $this->entityManager->getConnection()->fetchAll($sql, $params);
        return $cardapiosDatas;
    }
    
    public function retornaServicosCardapio($centrais, $filiais, $dtInicial, $dtFinal, $nrorg) {
        $cardapiosDatas = $this->retornaCardapiosDatas($centrais, $dtInicial, $dtFinal, $nrorg);
        $sqlServicos = "SELECT P.CDFILCENTPME, S.CDFILIAL, P.CDSERVFILI, S.NMSERVICO
                      FROM CARDIAPME P
                      JOIN SERVICO S ON S.CDFILIAL = :CDFILIAL AND S.CDSERVFILI = P.CDSERVFILI
                     WHERE INSTR(:CDFILCENTPME, P.CDFILCENTPME) <> 0
                     AND (P.NRCARDPME = :NRCARDPME) 
                       AND (P.CDFILIAL = P.CDFILCENTPME) 
                       AND DTCARDIAPME BETWEEN :DTINI AND :DTFIM
                       AND NOT EXISTS(SELECT CDFILIAL FROM CARDIAPME C 
                                       WHERE (C.CDFILCENTPME = P.CDFILCENTPME) AND (C.NRCARDPME = P.NRCARDPME) 
                                         AND (C.CDSERVFILI = P.CDSERVFILI) AND (C.CDFILIAL = :CDFILIAL) 
                                         AND (C.DTCARDIAPME = P.DTCARDIAPME) AND (C.IDCARDESPEC = 'S') )
                     UNION ALL 
                    SELECT P.CDFILCENTPME, S.CDFILIAL, P.CDSERVFILI, S.NMSERVICO
                      FROM CARDIAPME P
                      JOIN SERVICO S ON S.CDFILIAL = :CDFILIAL AND S.CDSERVFILI = P.CDSERVFILI
                     WHERE INSTR(:CDFILCENTPME, CDFILCENTPME) <> 0
                     AND (NRCARDPME = :NRCARDPME) 
                       AND DTCARDIAPME BETWEEN :DTINI AND :DTFIM
                        AND (P.CDFILIAL = :CDFILIAL) AND (IDCARDESPEC = 'S')";
                        
        if (is_string($filiais)) $filiais = explode('_', $filiais);
        $servicos = [];
        foreach($filiais as $filial) {
            foreach($cardapiosDatas as $cardapio) {
                $paramsServicos = array(
                    'CDFILIAL' => $filial,
                    'DTINI' => $dtInicial,
                    'NRCARDPME' => $cardapio['NRCARDPME'],
                    'CDFILCENTPME' => $centrais,
                    'DTFIM' => $dtFinal,
                );
                $servicos = array_merge($servicos, $this->entityManager->getConnection()->fetchAll($sqlServicos, $paramsServicos));
            }
        }
        return $this->buildResponseData($servicos);
    }

    public function retornaPratosCardapios($centrais, $filiais, $dtInicial, $dtFinal, $nrorg) {
        // $cardapiosDatas = $this->retornaCardapiosDatas($centrais, $dtInicial, $dtFinal, $nrorg);
        $sqlPratos = "SELECT CC.CDFILCENTPME, CC.NRCARDPME, CC.NRCARDIAPME, :CDFILIAL AS CDFILIAL, CD.CDSERVFILI,
                       TO_CHAR(CD.DTCARDIAPME, 'DD/MM/YYYY') AS DTCARDIAPME, CC.NRSEQPRACPME, CC.NRORDPRACPME, CC.CDFILAUXPRAT, CC.CDPRATO, PT.NMPRATO,
                       CC.CDTIPOPRAT, T.NMTIPOPRAT, CC.VRPECONSCPME, CD.IDTIPO, CDTIPOCARD, TO_CHAR(CD.DTCARDIAPME, 'DDMMYYYY') AS DTCARDIAPME_JOIN, CD.QTCOMESTCPME,
                       (CC.VRPECONSCPME * CD.QTCOMESTCPME) / 100 AS QTCOMENSAL,
                       CC.CDFILCENTPME || CC.NRCARDPME || CC.NRCARDIAPME || :CDFILIAL || CDTIPOCARD || CD.CDSERVFILI || TO_CHAR(CD.DTCARDIAPME, 'DDMMYYYY') || CC.CDPRATO AS KEY
                  FROM PRATO PT, COMPCARDPME CC, TIPOPRAT T, CONFCARDPME CF,
                       (SELECT 'P' IDTIPO, P.CDFILCENTPME, P.NRCARDPME, NRCARDIAPME, CDFILIAL, DTCARDIAPME, CDSERVFILI, CDTIPOCARD, P.QTCOMESTCPME
                          FROM CARDIAPME P 
                          JOIN CARDAPIOPME C ON C.CDFILCENTPME = P.CDFILCENTPME
                                                AND C.NRCARDPME = P.NRCARDPME
                                               AND (C.DTINICARDPME BETWEEN :DTINI AND :DTFIM
                                               OR C.DTFINCARDPME BETWEEN :DTINI AND :DTFIM
                                               OR :DTINI BETWEEN C.DTINICARDPME AND C.DTFINCARDPME
                                               OR :DTFIM BETWEEN C.DTINICARDPME AND C.DTFINCARDPME)                                               
                         WHERE INSTR(:CDFILCENTPME, P.CDFILCENTPME) <> 0
                           AND (P.CDFILIAL = P.CDFILCENTPME) 
    					   AND DTCARDIAPME BETWEEN :DTINI AND :DTFIM
                           AND NOT EXISTS(SELECT CDFILIAL FROM CARDIAPME C 
                                           WHERE (C.CDFILCENTPME = P.CDFILCENTPME) AND (C.NRCARDPME = P.NRCARDPME) 
                                             AND (C.CDSERVFILI = P.CDSERVFILI) AND (C.CDFILIAL = :CDFILIAL) 
                                             AND (C.DTCARDIAPME = P.DTCARDIAPME) AND (C.IDCARDESPEC = 'S') )
                         UNION ALL 
                        SELECT 'E' IDTIPO, CD.CDFILCENTPME, CD.NRCARDPME, NRCARDIAPME, CDFILIAL, DTCARDIAPME, CDSERVFILI, CDTIPOCARD, CD.QTCOMESTCPME
                          FROM CARDIAPME CD
                          JOIN CARDAPIOPME C ON C.CDFILCENTPME = CD.CDFILCENTPME
                                                AND C.NRCARDPME = CD.NRCARDPME
                                               AND (C.DTINICARDPME BETWEEN :DTINI AND :DTFIM
                                               OR C.DTFINCARDPME BETWEEN :DTINI AND :DTFIM
                                               OR :DTINI BETWEEN C.DTINICARDPME AND C.DTFINCARDPME
                                               OR :DTFIM BETWEEN C.DTINICARDPME AND C.DTFINCARDPME)
                         WHERE INSTR(:CDFILCENTPME, CD.CDFILCENTPME) <> 0
    					   AND DTCARDIAPME BETWEEN :DTINI AND :DTFIM
                            AND (CDFILIAL = :CDFILIAL) AND (IDCARDESPEC = 'S')) CD 
                   WHERE (CC.CDFILCENTPME = CD.CDFILCENTPME) AND (CC.NRCARDPME = CD.NRCARDPME) 
                     AND (CC.NRCARDIAPME = CD.NRCARDIAPME) AND (PT.CDFILAUXPRAT = CC.CDFILAUXPRAT) 
                     AND (PT.CDPRATO = CC.CDPRATO)
                     AND (CC.CDTIPOPRAT = T.CDTIPOPRAT)
                     AND (CF.CDFILCENTPME = CD.CDFILCENTPME)
                     AND (CF.NRCARDPME = CD.NRCARDPME)
                     AND (CF.CDSERVFILI = CD.CDSERVFILI)
                     AND (CF.CDTIPOPRAT = T.CDTIPOPRAT)
                     AND CF.IDIMPCNFCPME = 'S'
                     AND CC.NRSEQPRACPME = CF.NRSEQCNFCPME AND CC.NRORDPRACPME = CF.NRORDCNFCPME
                     ORDER BY 5, 7";
                     
                 /*(P.CDFILCENTPME = :CDFILCENTPME) */
                 /*(P.CDSERVFILI = :CDSERVFILI) AND */
                 /*(CDFILCENTPME = :CDFILCENTPME) */
                 /*AND (CDSERVFILI = :CDSERVFILI)*/
                 
        if (is_string($filiais)) $filiais = explode('_', $filiais);
        $pratosCardapios = [];
        foreach($filiais as $filial) {
            // foreach($cardapiosDatas as $cardapio) {
            $paramsPratos = array(
                'CDFILIAL' => $filial,
                'DTINI' => $dtInicial,
                // 'NRCARDPME' => $cardapio['NRCARDPME'],
                'CDFILCENTPME' => $centrais,
                'DTFIM' => $dtFinal,
            );
            $pratosCardapios = array_merge($pratosCardapios, $this->entityManager->getConnection()->fetchAll($sqlPratos, $paramsPratos));
            // }
        }
        
        $resultado = [];
        foreach($pratosCardapios as $prato) {
            $row = array(
                'KEY' => $prato['CDFILCENTPME'] . $prato['NRCARDPME'] . $prato['NRCARDIAPME'] . $prato['CDFILIAL'] . $prato['CDTIPOCARD'] . $prato['CDSERVFILI'] . $prato['DTCARDIAPME_JOIN'],
                'CDFILCENTPME' => $prato['CDFILCENTPME'],
                'NRCARDPME' => $prato['NRCARDPME'],
                'NRCARDIAPME' => $prato['NRCARDIAPME'],
                'CDFILIAL' => $prato['CDFILIAL'],
                'CDTIPOCARD' => $prato['CDTIPOCARD'],
                'CDSERVFILI' => $prato['CDSERVFILI'],
                'DTCARDIAPME' => $prato['DTCARDIAPME'],
            );
            if (isset($resultado[$row['KEY']])) {
                $resultado[$row['KEY']]['PRATOS'][] = array(
                    'KEY' => $prato['KEY'],
                    'CDPRATO' => $prato['CDPRATO'],
                    'NMPRATO' => $prato['NMPRATO'],
                    'NRSEQPRACPME'  => $prato['NRSEQPRACPME'],
                    'NRORDPRACPME' => $prato['NRORDPRACPME'],
                    'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                    'CDTIPOPRAT' => $prato['CDTIPOPRAT'],
                    'NMTIPOPRAT' => $prato['NMTIPOPRAT'],
                    'VRPECONSCPME' => $prato['VRPECONSCPME'],
                    'IDTIPO' => $prato['IDTIPO'],
                    'QTCOMESTCPME' => $prato['QTCOMESTCPME'],
                    'QTCOMENSAL' => $prato['QTCOMENSAL'],
                );
            } else {
                $row['PRATOS'][] = array(
                    'KEY' => $prato['KEY'],
                    'CDPRATO' => $prato['CDPRATO'],
                    'NMPRATO' => $prato['NMPRATO'],
                    'NRSEQPRACPME'  => $prato['NRSEQPRACPME'],
                    'NRORDPRACPME' => $prato['NRORDPRACPME'],
                    'CDFILAUXPRAT' => $prato['CDFILAUXPRAT'],
                    'CDTIPOPRAT' => $prato['CDTIPOPRAT'],
                    'NMTIPOPRAT' => $prato['NMTIPOPRAT'],
                    'VRPECONSCPME' => $prato['VRPECONSCPME'],
                    'IDTIPO' => $prato['IDTIPO'],
                    'QTCOMESTCPME' => $prato['QTCOMESTCPME'],
                    'QTCOMENSAL' => $prato['QTCOMENSAL'],
                );
                $resultado[$row['KEY']] = $row;
            }
        }
        
        return $this->buildResponseData(array_values($resultado));
    }
    
    public function retornaTiposCardapio($centrais, $dtInicial, $dtFinal, $nrorg) {
        $sql = "SELECT C.CDFILCENTPME, C.CDTIPOCARD, TC.NMTIPOCARD
          FROM CARDAPIOPME C
          JOIN TIPOCARD TC ON TC.CDFILCENTPME = C.CDFILCENTPME
          AND TC.CDTIPOCARD = C.CDTIPOCARD
         WHERE 
         INSTR(:CDFILCENTPME, C.CDFILCENTPME) <> 0
         AND C.NRORG = :NRORG
           AND (C.DTINICARDPME BETWEEN :DTINI AND :DTFIM
           OR C.DTFINCARDPME BETWEEN :DTINI AND :DTFIM
           OR :DTINI BETWEEN C.DTINICARDPME AND C.DTFINCARDPME
           OR :DTFIM BETWEEN C.DTINICARDPME AND C.DTFINCARDPME)";

        $params = array(
            'NRORG' => $nrorg,
            'CDFILCENTPME' => $centrais,
            'DTINI' => $dtInicial,
            'DTFIM' => $dtFinal,
        );
        // $cardapiosDatas = $this->entityManager->getConnection()->fetchAll($sql, $params);
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }
}