<?php
namespace SCM\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\Log\AbstractLogger;

class Log extends Listener{

    protected $logger;
    protected $routePaths = array('Login' => '/Login', 'Importacao' => '/Solicitacao', 'Exportacao' => '/Exportacao');
    protected $period = array('1' => '15' , '2' => '30', '3' => '15', '4' => '30');
    protected $separator = '                      ';

    public function __construct($logger) {
        $this->logger = $logger;
    }

    public function preDispatch(Request $request) {
        $message = "SchoolMeals: {requestContent}";

        $text = $this->returnTextToSave($request);

        if ($text) {

            $this->logger->debug($message, array(
                "requestContent" => var_export($text, true)
            ));
        }
        
    }

    private function returnTextToSave($request) {

        $routePath = $request->getRoutePath();
        $date = new \DateTime();
        $text = '';

        if ($this->routePaths['Login'] === $routePath) {

            $conditions = $request->getFilterCriteria()->getConditions();
            $text = "Login - " . $date->format('d/m/Y H:i:s') . "\n" . $this->separator .
                    "user: " . $conditions[0]['value'];

        } else if ($this->routePaths['Importacao'] === $routePath) {

            $text = $this->getTextGeneralParams($request->getFilterCriteria()->getConditions());

        } else if ($this->routePaths['Exportacao'] === $routePath) {

            $conditions = $request->getFilterCriteria()->getConditions();
            $text = "Exportação - " . $date->format('d/m/Y H:i:s') . "\n" . $this->separator .
                    "user: " . $conditions[0]['value'];
        }

        return $text;
    }

    private function getTextGeneralParams($conditions) {

        // $nrorg = $conditions[0]['value'];
        $nrorg = $this->environment->getNrorg();
        $filiaisImportacao = $conditions[1]['value'];
        $dataInicial = $conditions[2]['value'];
        $dataFinal = $conditions[3]['value'];
        $periodo = $conditions[4]['value'];
        $userMail = $conditions[5]['value'];
        $date = new \DateTime();

        $text = "Importação - " . $date->format('d/m/Y H:i:s') . "\n" . $this->separator .
                "user: " . $userMail . "\n" . $this->separator .
                "nrorg: " . $nrorg . "\n" . $this->separator .
                "unidades: " . implode(', ', $filiaisImportacao) . "\n" . $this->separator;

        switch ($periodo) {
            case '1':
                $text .= $this->subDays($this->period[$periodo]);
                break;
            
            case '2':
                $text .= $this->subDays($this->period[$periodo]);
                break;

            case '3':
                $text .= $this->addDays($this->period[$periodo]);
                break;

            case '4':
                $text .= $this->addDays($this->period[$periodo]);
                break;

            default:
                $text .= "data inicial: " . $dataInicial . "\n" . $this->separator . 
                         "data final: " . $dataFinal;
                break;
        }

        return $text;

    }

    private function subDays($days) {

        $tipo = 'D';

        $date = new \DateTime();
        $dataFinal = $date->format('d/m/Y');

        $date->sub(new \DateInterval("P$days$tipo"));
        $dataInicial = $date->format('d/m/Y');

        return "data inicial: " . $dataInicial . "\n" . $this->separator . 
               "data final: " . $dataFinal;

    }

    private function addDays($days) {

        $tipo = 'D';

        $date = new \DateTime();
        $dataInicial = $date->format('d/m/Y');

        $date->add(new \DateInterval("P$days$tipo"));
        $dataFinal = $date->format('d/m/Y');

        return "data inicial: " . $dataInicial . "\n" . $this->separator . 
               "data final: " . $dataFinal;

    }

}