import { Config } from '@zeedhi/core';
import { IExternalColumn, IRouteConfig } from '@zeedhi/zeedhi-miragejs';
import qs from 'qs';

const getFilial = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDOPERADOR: localStorage.getItem('CDOPERADOR') || '',
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || 'T',
  };
  const filial: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/filial?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'CDFILIAL',
    columns: {
      CDFILIAL: {},
      CDEMPRESA: {},
      CDFILCENTCMP: {},
      CDFILCENTPME: {},
      CDFILLANCA: {},
      DESCRICAO: {},
      NMFILIAL: {},
    },
  };
  return filial;
};

const getInventarios = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const inventarios: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/inventarios?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDFILIAL: {},
      DSINVENTRETI: {},
      DTINVENTRETI: {},
      NRINVENTRETI: {},
    },
  };
  return inventarios;
};

const getProdutos = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    CDGRUPPROD: localStorage.getItem('CDGRUPPROD') || '',
  };
  const produtos: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/produtos?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'CDPRODUTO',
    columns: {
      CDARVPROD: {},
      CDFILIAL: {},
      CDPRODUTO: {},
      NMPRODUTO: {},
      SGUNIDADE: {},
      TIPOPRODUTO: {},
    },
  };
  return produtos;
};

const getProdutosInventario = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    NRINVENTRETI: null,
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const produtosInventario: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/produtosInventario?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDARVPROD: {},
      CDFILIAL: {},
      CDOPERINCLUSAO: {},
      CDOPERULTATU: {},
      CDPRODUTO: {},
      DTINCLUSAO: {},
      DTULTATU: {},
      NMPRODUTO: {},
      NRINVENTRETI: {},
      NRITINVENTRETI: {},
      QTITINVENTRETI: {},
    },
  };
  return produtosInventario;
};

const getAF = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const af: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/af?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDFILDEST: {},
      CDFILLANC: {},
      NRAUTOFORN: {},
      CDPRODUTO: {},
      CDARVPROD: {},
      NMPRODUTO: {},
      SGUNIDADE: {},
      CDMARCA: {},
      NMMARCA: {},
      DTPROGENTAF: {},
      VRPERDESAF: {},
      QTPROGENTAF: {},
      VRPREUNIAF: {},
      VRDESCITMAF: {},
      VRTOTALITEM: {},
      NMRAZSOCFORN: {},
      NMFANTFORN: {},
    },
  };
  return af;
};

const getClientesDoServicoEfetivos = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
    PERIODO: 5,
    CDSERVFILI: 'T',
    CDTIPOCARD: 'T',
  };
  const clientesDoServicoEfetivos: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/clientesDoServicoEfetivos?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDFILCENTPME: {},
      CDTIPOCARD: {},
      CDFILIAL: {},
      CDSERVFILI: {},
      CDCLIENTE: {},
      NMRAZSOCCLIE: {},
      DTEFETCLPME: {},
      QTCOMEFCLPME: {},
      QTFUNEFCLPME: {},
      QTDOAEFCLPME: {},
      TOTAL: {},
      IDCLIENPRIN: {},
      DTFINAL: {},
      DTINICIAL: {},
    },
  };
  return clientesDoServicoEfetivos;
};

const getServicosCardapioEfetivos = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const servicosCardapioEfetivos: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/servicosCardapioEfetivos?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDTIPOCARD: {},
      NMTIPOCARD: {},
      DSCARDPME: {},
      CDFILCENTPME: {},
      NRCARDPME: {},
      NRCARDIAPME: {},
      CDSERVFILI: {},
      CDFILIAL: {},
      DATA_EXTENSO: {},
      DTCARDIAPME: {},
      IDCARDESPEC: {},
      NMSERVICO: {},
      QTCOMEFECPME: {},
      QTCLIEFECPME: {},
      QTFUNEFECPME: {},
      QTDOAEFECPME: {},
    },
  };
  return servicosCardapioEfetivos;
};

const getUltimoInventario = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
  };
  const ultimoInventario: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/ultimoInventario?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDFILIAL: {},
      NRINVENTRETI: {},
      NRITINVENTRETI: {},
      CDPRODUTO: {},
      NMPRODUTO: {},
      CDARVPROD: {},
    },
  };
  return ultimoInventario;
};

const getJustificativas = (): IRouteConfig<IExternalColumn> => {
  const justificativas: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/justificativas`,
    dataColumn: 'data',
    uniqueKey: 'CDJUSTSOL',
    columns: {
      CDJUSTSOL: {},
      DSJUSTOPER: {},
      DSJUSTSOLEXT: {},
    },
  };
  return justificativas;
};

const getProdutosSolicitacao = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const produtosSolicitacao: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/produtosSolicitacao?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDFILICONCATNRSOL: {},
      CDFILLAN: {},
      CDFILSOLI: {},
      NRSOLICMP: {},
      CDPRODUTO: {},
      CDARVPROD: {},
      NMPRODUTO: {},
      SGUNIDADE: {},
      QTSOLIPROD: {},
      DTPROGENTR: {},
      CDFILIOPELOG: {},
      IDSOLIAPROV: {},
      QTAJUSTADA: {},
      DSJUSTOPER: {},
      IDITMSOLEXTRA: {},
      DSJUSTSOLEXT: {},
      CDFILCENT: {},
      CDFILLANCA: {},
      CDOPERLANC: {},
      DTHRLANCSOL: {},
      DTSOLCMP: {},
      DTUTILIZA: {},
      IDSOLEXTRA: {},
      IDTIPLAN: {},
      QTITEMSOLIC: {},
      CDFILIAL: {},
    },
  };
  return produtosSolicitacao;
};

const getParametros = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
  };
  const parametros: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/parametros?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'CDFILIAL',
    columns: {
      CENTRAL: {},
      CDFILIAL: {},
      IDBLOQALTSOLAVR: {},
      IDEXPSOLCMP: {},
      IDUTILALMOX: {},
      IDUTILLOTE: {},
      IDUTPRODPREF: {},
      INCLUIR_PRODUTO_APROVADO: {},
      ALTERAR_PRODUTO_APROVADO: {},
      DESAPROVAR_PRODUTO_ALTERADO: {},
      IDUTAPROVSOL: {},
    },
  };
  return parametros;
};

const getProximoCodigoInventario = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
  };
  const proximoCodigoInventario: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/proximoCodigoInventario?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'CDFILIAL',
    columns: {
      CDFILIAL: {},
      PROXIMOCODIGO: {},
    },
  };
  return proximoCodigoInventario;
};

const getGruposProduto = (): IRouteConfig<IExternalColumn> => {
  const gruposProduto: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/gruposProduto`,
    dataColumn: 'data',
    uniqueKey: 'CDGRUPPROD',
    columns: {
      CDGRUPPROD: {},
      NMGRUPPROD: {},
    },
  };
  return gruposProduto;
};

const getPratosCardapios = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILCENTPME: localStorage.getItem('CENTRAIS_IMPORTACAO') || '',
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const pratosCardapios: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/pratosCardapios?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      KEY: {},
      CDFILCENTPME: {},
      NRCARDPME: {},
      NRCARDIAPME: {},
      CDFILIAL: {},
      CDSERVFILI: {},
      DTCARDIAPME: {},
      CDTIPOCARD: {},
      PRATOS: {},
    },
  };
  return pratosCardapios;
};

const getTiposCardapio = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILCENTPME: localStorage.getItem('CENTRAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const tiposCardapio: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/tiposCardapio?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      CDFILCENTPME: {},
      CDTIPOCARD: {},
      NMTIPOCARD: {},
    },
  };
  return tiposCardapio;
};

const getServicosCardapio = (): IRouteConfig<IExternalColumn> => {
  const filter = {
    CDFILCENTPME: localStorage.getItem('CENTRAIS_IMPORTACAO') || '',
    CDFILIAL: localStorage.getItem('FILIAIS_IMPORTACAO') || '',
    DTINICIAL: localStorage.getItem('DATA_INICIAL') || '',
    DTFINAL: localStorage.getItem('DATA_FINAL') || '',
  };
  const servicosCardapio: IRouteConfig<IExternalColumn> = {
    externalApi: true,
    persist: true,
    apiUrl: `${Config.endPoint.replace('//backend', '/backend')}/servicosCardapio?${qs.stringify(filter)}`,
    dataColumn: 'data',
    uniqueKey: 'KEY',
    columns: {
      CDFILCENTPME: {},
      CDFILIAL: {},
      CDSERVFILI: {},
      NMSERVICO: {},
    },
  };
  return servicosCardapio;
};

const config = () => ({
  filial: getFilial(),
  inventarios: getInventarios(),
  produtos: getProdutos(),
  produtosInventario: getProdutosInventario(),
  af: getAF(),
  clientesDoServicoEfetivos: getClientesDoServicoEfetivos(),
  servicosCardapioEfetivos: getServicosCardapioEfetivos(),
  ultimoInventario: getUltimoInventario(),
  justificativas: getJustificativas(),
  produtosSolicitacao: getProdutosSolicitacao(),
  parametros: getParametros(),
  proximoCodigoInventario: getProximoCodigoInventario(),
  gruposProduto: getGruposProduto(),
  pratosCardapios: getPratosCardapios(),
  tiposCardapio: getTiposCardapio(),
  servicosCardapio: getServicosCardapio(),
});

const getConfig = (force: boolean = false) => {
  if (force) return config();
  if (!localStorage.getItem('FILIAIS_IMPORTACAO')) return {};
  return config();
};

export { getConfig };
